/*
 *  html.c
 *  mod_musicindex
 *
 *  $Id: html.c,v 1.58 2004/01/31 18:07:06 boudinr Exp $
 *
 *  Created by Thibaut VARENE on Thu Mar 20 2003.
 *  Copyright (c) 2003-2004 Regis BOUDIN
 *  Copyright (c) 2003-2004 Thibaut VARENE
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */

/**
 * @file 
 * HTML I/O handling.
 *
 * @author Regis Boudin
 * @author Thibaut Varene
 * @version $Revision: 1.58 $
 * @date 2003-2004
 *
 * That file groups all functions related to HTML generation
 * and parsing, that is to say functions that are used to
 * send HTML code to the client and functions that are in
 * the process of parsing URI requests.
 * Here are also the functions related to m3u playlist creation.
 */
 
#include "html.h"
#include "inf.h"

/**
 * Formats the list of songs to be sent.
 *
 * This function formats a list of tracks (if any) to be sent
 * to the client, as found in the p list passed in argument.
 * Depending on config options, it allows streaming/shuffle etc.
 * 
 * @param r Apache request_rec struct to handle connection details.
 * @param p struct mu_ent.
 * @param conf MusicIndex configuration paramaters struct.
 */
static void list_songs(request_rec *r, struct mu_ent *p, mu_config *conf)
{
	struct mu_ent *q = p;
	unsigned short i = 0, j = 0;
	char *current = "*", *new = NULL;
	
	while (q) {
		if ((conf->options & (MI_SEARCH | MI_RECURSIVE)) == (MI_SEARCH | MI_RECURSIVE)) {
			new = ap_make_dirstr_parent(r->pool, q->file);
		
			if (strcmp(current, new)) {
				current = new;
				if ((current[0] == '\0') || (current[1] == '\0'))
					ap_rvputs(r, " <tr class=\"title\"><th align=\"left\" colspan=\"10\">",
						_("in Current Directory"), "</th>\n", NULL);
				else
					ap_rvputs(r, " <tr class=\"title\"><th align=\"left\" colspan=\"10\">in <a href=\"",
						current, "\">", current, "</a></th>\n", NULL);
				j = 0;
			}
		}

		if ((j++) & 1)
			ap_rputs(" <tr class=\"odd\">\n", r);
		else
			ap_rputs(" <tr class=\"even\">\n", r);

		if (conf->options & (MI_ALLOWDWNLD | MI_ALLOWSTREAM)) {		
			ap_rputs("  <td class=\"Select\">\n", r);
		
			if (conf->options & MI_ALLOWSTREAM)	/* Display checkbox */
				ap_rvputs(r, "   <input type=\"checkbox\" name=\"file\" value=\"",
					q->file, "\" />\n", NULL);
		
			if (conf->options & MI_ALLOWDWNLD)	/* Display [fetch] */
				ap_rvputs(r, "   <a href=\"", ap_escape_uri(r->pool, q->file), "\">"
					"<img alt=\"[F]\" src=\"", conf->fetch_icon, "\" /></a>\n", NULL);
		
			if (conf->options & MI_ALLOWSTREAM)	/* Display [stream] */
				ap_rvputs(r, "   <a href=\"", ap_escape_uri(r->pool, q->file), "?stream\">"
					"<img alt=\"[S]\" src=\"", conf->sound_icon, "\" /></a>\n", NULL);

			ap_rputs("  </td>\n", r);
		}
		
		for (i = 0; conf->fields[i] != '\0'; i++) {
			switch (conf->fields[i]) {
				case SB_TITLE:
					ap_rvputs(r, "  <td class=\"Title\">", q->title, "</td>\n", NULL);
					break;
				case SB_TRACK:
					if (q->track)
						ap_rprintf(r, "  <td class=\"Track\">%d</td>\n", q->track);
					else
						ap_rputs( "  <td></td>\n", r);
					break;
				case SB_POSN:
					if (q->posn)
						ap_rprintf(r, " <td class=\"Disc\">%d</td>\n", q->posn);
					else
						ap_rputs( "  <td></td>\n", r);
					break;
				case SB_ARTIST:
					ap_rprintf(r, " <td class=\"Artist\">%s</td>\n", q->artist ? q->artist : "");
					break;
				case SB_LENGTH:
					if (conf->options & MI_QUICKPL)
						break;
					if (q->length)
						ap_rprintf(r, "  <td class=\"Length\">%ld:%.2ld</td>\n", q->length / 60, q->length % 60);
					else
						ap_rputs( "  <td></td>\n", r);
					break;
				case SB_BITRATE:
					if (conf->options & MI_QUICKPL)
						break;
					if (q->bitrate)
						ap_rprintf(r, "  <td class=\"Bitrate\">%ld</td>\n", q->bitrate >> 10);
					else
						ap_rputs( "  <td></td>\n", r);
					break;
				case SB_DATE:
					if (q->date)
						ap_rprintf(r, "  <td class=\"Date\">%d&nbsp;</td>\n", q->date);
					else
						ap_rputs( "  <td></td>\n", r);
					break;
				case SB_ALBUM:
					ap_rprintf(r, " <td class=\"Album\">%s</td>\n", q->album ? q->album : "");
					break;
				case SB_FILETYPE:
					switch(q->filetype) {
						case FT_MP3:
							ap_rvputs(r, "  <td>MP3</td>\n", NULL);
							break;
						case FT_OGG:
							ap_rvputs(r, "  <td>OGG</td>\n", NULL);
							break;
					}
					break;
				case SB_GENRE:
					ap_rprintf(r, " <td class=\"Genre\">%s</td>\n", q->genre ? q->genre : "");
					break;
			}
		}
		ap_rputs(" </tr>\n", r);
		q = q->next;
	}

}

/**
 * Treats the URL arguments.
 *
 * This function searches for keywords passed as URL arguments (with "?xxx")
 * and sets the handler options accordingly.
 * 
 * @param r Apache request_rec struct to handle connection details.
 * @param conf MusicIndex configuration paramaters struct.
 *
 * @return The corresponding option letter.
 */
char treat_args(request_rec *r, mu_config *conf)
{
	const char *s = r->args;
	const char *p;
	unsigned short i;

	if ((s == NULL) || (*s == '\0'))
		return 'A';

	while (s[0]) {
		p = ap_getword(r->pool, &s, '&');
		if (!strncmp(p, "action=", 7)) {
			p += 7;
			if ((!strcmp(p, "playlist") || !strcmp(p, "Play+Selected")) && (conf->options & MI_ALLOWSTREAM)) {
				return 'L';
			} else if ((!strcmp(p, "playall") || !strcmp(p, "Play+All")) && (conf->options & MI_ALLOWSTREAM)) {
				conf->options |= MI_PLAYLIST; /* generate a playlist */
				return 'A';
			} else if (!strcmp(p, "Shuffle+All") && (conf->options & MI_ALLOWSTREAM)) {
				conf->order[0] = SB_RANDOM;
				conf->order[1] = SB_DEFAULT;
				set_fctptrs(conf);
				conf->options |= MI_PLAYLIST; /* generate a playlist */
				return 'A';
			} else if (!strcmp(p, "Search") && (conf->options & MI_ALLOWSEARCH) && (conf->search[0] != '\0')) {
				conf->options |= MI_SEARCH;
				conf->order[0] = SB_URI;
				set_fctptrs(conf);
				if (!conf->cache_path)
					conf->options |= MI_QUICKPL;
			} else if (!strcmp(p, "Recursive+Search") && (conf->options & MI_ALLOWSEARCH) && (conf->search[0] != '\0')) {
				conf->play_recursive |= MI_RECURSIVE;
				conf->order[0] = SB_DIR;
				conf->order[1] = SB_URI;
				set_fctptrs(conf);
				return 'A';
			}	
		}
		else if (!strncmp(p, "sort=", 5)) {
			p += 5;
			for (i = ARG_NUMBER-strlen(p)-1; i != 0; i--)
				conf->order[i] = conf->order[i-strlen(p)];
			for (i = 0; (i<strlen(p)) && (i<ARG_NUMBER); i++)
				conf->order[i] = p[i];
			conf->order[ARG_NUMBER-1] = SB_DEFAULT;
			set_fctptrs(conf);
		}
		else if (!strncmp(p, "option=", 7)) {
			p += 7;
			if (!strcmp(p, "recursive"))
				conf->play_recursive |= MI_RECURSIVE;
			else if (!strcmp(p, "shuffle")) {
				conf->order[0] = SB_RANDOM;
				conf->order[1] = SB_DEFAULT;
				set_fctptrs(conf);
			}
			else if (!strcmp(p, "quick"))
				conf->options |= MI_QUICKPL;
			else {
				conf->search = ap_pstrdup(r->pool, p);
				for (i=0; p[i]; i++) {
					if (p[i] == '+')
						conf->search[i] = ' ';
				}
				ap_unescape_url(conf->search);
			}
		}
	}
	return 'A';
}

/**
 * Formats a complete url string to be sent to client.
 *
 * This function does all the URL formatting, in the form:
 * http://[user:][passwd\@]hostname[:port]/uri/
 * 
 * @param r Apache request_rec struct to handle connection details.
 * @param uri A string (basically a path) to be sent to form a complete url.
 * @param command An eventual command (like ?action=playall).
 * @param conf MusicIndex configuration paramaters struct.
 */
static void send_url(request_rec *r, char *uri, char *command, mu_config *conf)
{
	char prefix[MAX_PREFIX];
	char str_port[6];	/* 65536 + '\0' */
	char *bufcoded, *decoded;
	unsigned short l;
	
	strcpy(prefix, "http://");

	/* if we have an icecast server, it takes precedence, in a 'staticdir' fashion */
	if (conf->iceserver)
		strcat(prefix, conf->iceserver);
	else {
		if (REQUEST_USER(r)) {
			/* grab the auth credentials base64 encoded */
			const char *auth = ap_table_get(r->headers_in, "Authorization");
			if (auth) {
				bufcoded = strrchr(auth, ' ');
				bufcoded++;
				decoded = (char *)ap_palloc(r->pool, 1 + ap_base64decode_len(bufcoded));
				l = ap_base64decode(decoded, bufcoded);
				strncat(prefix, decoded, l);	/* we have "user:pass" */
			}
			strcat(prefix, "@");
		}
			
		/* add the hostname */
		strcat(prefix, r->hostname);
		
		/* add the port number if needed */
		if (r->server->port != 80) {
			sprintf(str_port, "%u", r->server->port);
			strcat(prefix, ":");
			strcat(prefix, str_port);
		}
	}
	
	/* add the uri and potential command */
	ap_rvputs(r, prefix, ap_escape_uri(r->pool, uri), NULL);
	if (command)
		ap_rvputs(r, command, NULL);
	ap_rvputs(r, "\n", NULL);
}	
		
/**
 * Sends HTML page headers and top of the page.
 *
 * This function takes care of the formating of the HTML page headers.
 * It sends the data that is common to all pages generated by the handler.
 * 
 * @param r Apache request_rec struct to handle connection details.
 * @param conf MusicIndex configuration paramaters struct.
 */
void send_head(request_rec *r, mu_config *conf)
{
	char *s, *t, *u;
	FILE *in = NULL;

	s = ap_pstrdup(r->pool, r->uri);

	ap_rvputs(r, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n"
		"<html>\n"
		"<head>\n"
		" <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n"
		" <meta name=\"generator\" content=\"mod_musicindex\">\n"
		" <link rel=\"stylesheet\" type=\"text/css\" href=\"", conf->css, "\">\n"
		" <link rel=\"shortcut icon\" href=\"", conf->favicon, "\">\n"
		" <title>", _("Musical index of"), " ", r->uri, "</title>\n"
		"</head>\n"
		"<body>\n", NULL);

	u = s + strlen(s);
	if (strlen(s) > 1)
		u--;
	if (*u == '/')
		*u = '\0';

	ap_rputs("<table id=\"header\">\n"
		" <tr>\n"
		"  <td id=\"mainicon\">\n"
		"   <img alt=\"Dir\" src=\"", r);

	if ((in = fopen(ap_pstrcat(r->pool, r->filename, "/", "cover.png", NULL), "r"))) {
		ap_rputs("cover.png", r);
		fclose(in);
	}
	else if ((in = fopen(ap_pstrcat(r->pool, r->filename, "/", "cover.jpg", NULL), "r"))) {
		ap_rputs("cover.jpg", r);
		fclose(in);
	}
	else if ((in = fopen(ap_pstrcat(r->pool, r->filename, "/", "cover.gif", NULL), "r"))) {
		ap_rputs("cover.gif", r);
		fclose(in);
	}
	else {
		ap_rputs(conf->cd_icon, r);
	}

	ap_rputs("\" />\n"
		"  </td>\n", r);

	t = u = s + 1;

	ap_rvputs(r, "  <td id=\"maintitle\">\n"
		"    <h1>\n"
		"     <a href=\"/\">", conf->title, "</a>\n", NULL);

	do {
		while ((*u != '/') && (*u != '\0'))
			u++;

		if (u != s + 1)
			ap_rvputs(r, "     <img src=\"", conf->arrow,
				"\" alt=\"=>\" />\n", NULL);

		*u = '\0';
		ap_rvputs(r, "     <a href=\"", ap_escape_uri(r->pool, s), "/\">", t, "</a>\n", NULL);
		*u++ = '/';
		t = u;
	} while (*u != '\0');

	ap_rputs("    </h1>\n", r);

	if (conf->options & MI_ALLOWSTREAM) {
		ap_rvputs(r, "    <a href=\"?option=recursive&amp;option=shuffle&amp;action=playall\">[",
			_("Shuffle All"), "]</a>\n", NULL);
		ap_rvputs(r, "    <a href=\"?option=recursive&amp;action=playall\">[",
			_("Stream All"), "]</a>\n", NULL);
	}
	
	ap_rputs("  </td>\n", r);

	/* displays a search box if option is activated */
 	if (conf->options & MI_ALLOWSEARCH){
		ap_rvputs(r,
			"  <td id=\"search\">\n"
			"   <form method=\"get\" action=\"", ap_escape_uri(r->pool, r->uri), "\""
			"enctype=\"application/x-www-form-urlencoded\" name=\"searching\">\n"
			"    <p>\n"
			"     <input type=\"text\" name=\"option\" />\n"
			"     <br>\n"
			"     <input type=\"hidden\" name=\"action\" value=\"Search\" />\n"
			"     <input type=\"submit\" name=\"action\" value=\"Search\" />\n"
			"     <input type=\"submit\" name=\"action\" value=\"Recursive Search\" />\n"	/* XXX youpi on peut pas i18n-iser... */
			"    </p>\n"
			"   </form>\n"
			"  </td>\n",
			NULL);
	}

	ap_rputs(" </tr>\n"
		"</table>\n"
		"<hr>\n<!-- begin main -->\n\n", r);

}

/**
 * Sends directory listing for the current folder.
 *
 * This function takes care of preparing and sending to the client the
 * list (if any) of the available directories in the current folder.
 * Depending on config options, it allows streaming/shuffle etc.
 * 
 * @param r Apache request_rec struct to handle connection details.
 * @param p struct mu_ent.
 * @param conf MusicIndex configuration paramaters struct.
 */
void send_directories(request_rec *r, struct mu_ent *p, mu_config *conf)
{
	struct mu_ent *q = p;
	unsigned short dircnt = 1, nb = 0;
	char *c;
	char temp[MAX_STRING];

	if (!q)
		return;

	if (q->filetype != FT_DIR)
		return;
	
	/* We count the number of directories for later use */
	do {
		nb++;
	} while ((q = q->next) && (q->filetype == FT_DIR));
	
	q = p;
		
	ap_rputs("<!-- begin subdirs -->\n<h2>",r);
	ap_rprintf(r, _("Music Directories (%d)"), nb);
	ap_rputs("</h2>\n\n<table id=\"directories\">\n", r);

	while (q && (q->filetype == FT_DIR)) {
		c = ap_cpystrn(temp, q->file, MAX_STRING)-1;
		*c = '\0';

		if (dircnt == 1)
			ap_rputs(" <tr>\n", r);

		ap_rvputs(r, "  <td>\n"
			"   <table>\n"
			"    <tr>\n"
			"     <td>\n"
			"      <a href=\"", ap_escape_uri(r->pool, q->file), NULL);

		if (conf->options & MI_ALLOWSTREAM)
			ap_rputs("?option=recursive&amp;action=playall", r);

		ap_rvputs(r, "\"><img "
			"alt=\"\" "
			"src=\"", conf->small_cd_icon, "\" /></a>\n"
			"     </td>\n", NULL);

		ap_rvputs(r, "     <td>\n"
			"      <a class=\"subdir\" href=\"", ap_escape_uri(r->pool, q->file), "\">",
	    		temp, "</a><br>\n", NULL);

		if (conf->options & MI_ALLOWSTREAM) {
			ap_rvputs(r, "      <a class=\"subdirbuttons\" href=\"",
				ap_escape_uri(r->pool, q->file), "?option=recursive&amp;option=shuffle&amp;action=playall\">[",
				_("Shuffle"), "]</a>\n", NULL);

			ap_rvputs(r, "      <a class=\"subdirbuttons\" href=\"",
				ap_escape_uri(r->pool, q->file), "?option=recursive&amp;action=playall\">[",
				_("Stream"), "]</a>\n", NULL);
		}
	
		ap_rputs("     </td>\n"
			"    </tr>\n"
			"   </table>\n"
			"  </td>\n",r);

		if (dircnt == DIRPERLINE) {
			dircnt = 0;
			ap_rputs(" </tr>\n", r);
		}
		dircnt++;
		q = q->next;
	}

	ap_rputs("</table>\n<hr>\n<!-- end subdirs -->\n\n", r);
}

/**
 * Sends track listing for the current folder.
 *
 * This function takes care of preparing and sending to the client the
 * list (if any) of the available songs in the current p list.
 * Depending on config options, it allows streaming/shuffle etc.
 *
 * @bug Know potential problem when sorting search results.
 * 
 * @param r Apache request_rec struct to handle connection details.
 * @param p struct mu_ent.
 * @param conf MusicIndex configuration paramaters struct.
 */
void send_tracks(request_rec *r, struct mu_ent *p, mu_config *conf)
{
	struct mu_ent *q = p;
	unsigned short i = 0, nb = 0;
	char *myargs = NULL;
	
	/* We count the number of songs for later use */
	for (q=p; (q!=NULL); q = q->next) {
		if (q->filetype != FT_DIR)
			nb++;
	}

	/* Go to the first song entry (just after dirs) */
	q = p;
	while (q) {
		if (q->filetype != FT_DIR)
			break;
		q = q->next;
	}

	if (q == NULL)
		return;

	ap_rputs("<!-- begin tracks -->\n<h2>", r);
	
	if (conf->options & MI_SEARCH)
		ap_rprintf(r, _("Result List (%d)"), nb);
	else
		ap_rprintf(r, _("Song List (%d)"), nb);
	
	ap_rputs("</h2>\n\n", r);

	ap_rvputs(r, "<form method=\"get\" action=\"", ap_escape_uri(r->pool, r->uri), "\" "
		"enctype=\"application/x-www-form-urlencoded\" name=\"form\">\n", NULL);
	
	ap_rputs("<table width=\"100%\" cellspacing=\"0\" border=\"0\">\n", r);

	ap_rputs(" <tr class=\"title\">\n", r);
	
	if (conf->options & (MI_ALLOWDWNLD | MI_ALLOWSTREAM))
		ap_rvputs(r, "  <th id=\"Select\">", _("Select"), "</th>\n", NULL);
	
	/* XXX HACK. This is rather ugly, because if the user changes the URI by hand, he might
	 * end up dicarding that piece of code and lead to unexpected (though not insecure ;o)
	 * behaviour.
	 * The basic idea is to avoid multiple "&sort=" requests in search URI. */
	if (conf->options & MI_SEARCH) {
		if (!(strncmp(strrchr(r->args, '&'), "&sort=", 6)))
			myargs = ap_pstrndup(r->pool, r->args, strlen(r->args)-7);
		else
			myargs = ap_pstrdup(r->pool, r->args);
		myargs = ap_pstrcat(r->pool, myargs, "&", NULL);
		/* we should escape the string but ap_escape_uri() doesn't work */
	}
	
	for (i = 0; conf->fields[i] != '\0'; i++) {	/* Display title line */
		switch (conf->fields[i]) { 
			case SB_TITLE:
				ap_rprintf(r, "  <th id=\"Title\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_TITLE, _("Title"));
				break;
			case SB_TRACK:
				ap_rprintf(r, "  <th id=\"Track\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_TRACK, _("Track"));
				break;
			case SB_POSN:
				ap_rprintf(r, "  <th id=\"Disc\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_POSN, _("Disc"));
				break;
			case SB_ARTIST:
				ap_rprintf(r, "  <th id=\"Artist\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_ARTIST, _("Artist"));
				break;
			case SB_LENGTH:
				if (!(conf->options & MI_QUICKPL))
					ap_rprintf(r, "  <th id=\"Length\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_LENGTH, _("Length"));
				break;
			case SB_BITRATE:
				if (!(conf->options & MI_QUICKPL))
					ap_rprintf(r, "  <th id=\"Bitrate\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_BITRATE, _("Bitrate"));
				break;
			case SB_ALBUM:
				ap_rprintf(r, "  <th id=\"Album\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_ALBUM, _("Album"));
				break;
			case SB_DATE:
				ap_rprintf(r, "  <th id=\"Date\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_DATE, _("Date"));
				break;
			case SB_FILETYPE:
				ap_rprintf(r, "  <th id=\"Filetype\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_FILETYPE, _("Type"));
				break;
			case SB_GENRE:
				ap_rprintf(r, "  <th id=\"Genre\"><a href=\"?%ssort=%c\">%s</a></th>\n", (conf->options & MI_SEARCH) ? myargs : "", SB_GENRE, _("Genre"));
				break;
		}
	}
	ap_rputs(" </tr>\n", r);

	list_songs(r, q, conf);
	
	if (conf->options & MI_SEARCH)
		ap_rputs("<tr class=\"title\"><th align=\"left\" colspan=\"10\">"
			"<input type=\"checkbox\" name=\"all\" onClick=\"for(var i=0;i<this.form.elements.length;i++){var inpt=this.form.elements[i];"
			"var m=inpt.name.match(/-/g);if((inpt.name.substr(0,4)=='file') && (m<1)) inpt.checked=this.form.all.checked}\" />\n"
			"Select All</th>", r);
			
	ap_rputs("</table>\n", r);

	ap_rvputs(r, "<p>\n"
/*		"<!--<input type=\"submit\" name=\"Add to Playlist\" value=\"Add to Playlist\" class=\"playlist\">-->"
		"<!--<input type=\"submit\" name=\"Add All to Playlist\" value=\"Add All to Playlist\" class=\"playlist\">-->" */
		" <input type=\"hidden\" name=\"sort\" value=\"", conf->order,"\" />\n"
		" <input type=\"submit\" name=\"action\" value=\"Play Selected\" />\n", NULL);	/* XXX i18n DTC */
	
	if (!(conf->options & MI_SEARCH))
		ap_rputs(" <input type=\"submit\" name=\"action\" value=\"Shuffle All\" />\n"
			" <input type=\"submit\" name=\"action\" value=\"Play All\" />\n", r);	/* XXX idem */
	
	ap_rputs("</p>\n</form>\n<hr>\n<!-- end tracks -->\n\n", r);
}

/**
 * Sends playlist with url and EXTM3U info to the client.
 *
 * This function prepares an m3u playlist (with extra information where
 * available: length, artist, title, album) and sends it to the client.
 *
 * @param r Apache request_rec struct to handle connection details.
 * @param p struct mu_ent.
 * @param conf MusicIndex configuration paramaters struct.
 */
void send_playlist(request_rec *r, mu_ent *p, mu_config *conf)
{
	mu_ent *q = p;
	if (!p)
		return;

	ap_rputs("#EXTM3U\n", r);

	while (q)
	{
		ap_rprintf(r, "#EXTINF:%li,", q->length);
		if (q->artist)
			ap_rvputs(r, q->artist, " - ", NULL);
		ap_rvputs(r, q->title, NULL);
		if (q->album)
			ap_rvputs(r, " (", q->album, ")", NULL);
		ap_rvputs(r, "\n", NULL);
		send_url(r, q->uri, NULL, conf);
		q = q->next;
	}
}

/**
 * Sends HTML page footers.
 *
 * This function takes care of the formating of the HTML page footers.
 * It sends the data that is common to all pages generated by the handler.
 * Currently it relies on apache_mp3 original stylesheets.
 * 
 * @param r Apache request_rec struct to handle connection details.
 * @param conf MusicIndex configuration paramaters struct.
 */
void send_foot(request_rec *r, mu_config *conf)
{
	ap_rprintf(r, "<!-- end main -->\n<!-- mod_musicindex v.%s -->\n", MUSIC_VERSION_STRING);
	ap_rprintf(r, "<!-- Authors: %s -->\n", MUSIC_AUTHORS_STRING);
	ap_rputs("<table border=\"0\" width=\"100%\">\n"
		" <tr>\n"
		"  <td align=\"left\">\n"
		"   <p>\n"
#if 1
		"    <a href=\"http://validator.w3.org/check/referer\">\n"
#else
		"    <a href=\"http://localhost/w3c-markup-validator/check/referer\">\n"
#endif
		"     <img src=\"http://www.w3.org/Icons/valid-html401\"\n"
        	"      alt=\"Valid HTML 4.01!\" height=\"31\" width=\"88\" />\n"
		"    </a>\n"
		"   </p>\n"
		"  </td>\n", r);
	ap_rprintf(r, "  <td align=\"right\">"
		"<a href=\"http://freshmeat.net/projects/musicindex/\">"
		"MusicIndex v.%s</a></td>\n", MUSIC_VERSION_STRING);
	ap_rputs(" </tr>\n"
		"</table>\n\n</body>\n</html>", r);
}
