/*
 *  playlist.c
 *  mod_musicindex
 *
 *  $Id: playlist.c,v 1.71 2004/02/08 21:33:31 varenet Exp $
 *
 *  Created by Thibaut VARENE on Thu Mar 20 2003.
 *  Copyright (c) 2003-2004 Regis BOUDIN
 *  Copyright (c) 2003-2004 Thibaut VARENE
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */

/**
 * @file 
 * Playlist management system.
 *
 * @author Regis Boudin
 * @author Thibaut Varene
 * @version $Revision: 1.71 $
 * @date 2003-2004
 *
 * This is where the magic takes place. Here are the functions in charge of
 * analyzing the content of the folders requested by the client, and of
 * generating the structures and lists that will be used by the HTML
 * subsystem to generate the pretty output.
 *
 * @todo Find out if we trash "detect by content".
 */

#include "playlist.h"
#include "html.h"

#include "cache.h"
#include "playlist-ogg.h"
#include "playlist-mp3.h"

/** mime types handled */
const char *handlers[] = {
	"audio/mpeg",
	"application/ogg",
	"audio/x-ogg",
	NULL
};

/**
 * Function pointer type, as we need it.
 *
 * Here comes the declaration of the funny part... We will (again) use
 * function pointers which will allocate a new entry  from the apache pool,
 * fill it, and close the "in" file if they recognise the file.
 */
typedef mu_ent* (*make_entry_ptr)(apr_pool_t*, mu_ent *, FILE *,
				mu_config *, mu_ent_names *, request_rec *);

/**
 * Function pointers array.
 *
 * Of course, as we defined function pointers, we will use them in a nice
 * array. They will be called in a sequential manner.<br>
 * make_cache_entry() <b>MUST</b> be called first.<br>
 * <i>Note</i> : a make_dir_entry() function to handle directories has 
 * already been tried, unfortunately, it implies some bad side effects.
 * Plus, it probably does not make things any quicker and increases the
 * number of function calls. (that's a personnal NB to the developpers,
 * to avoid repeating twice the same havoc as time goes ;o)
 */
static make_entry_ptr make_entry[] = {
#ifndef NO_CACHE
	make_cache_entry,
#endif
	make_ogg_entry,
	make_mp3_entry,
	NULL
};

/**
 * Creates a new musical entry (mp3, ogg or directory).
 *
 * @param pool Apache pool linked to the current request
 * @param head music entry structure which will follow this one in the list
 *
 * @return A new mu_ent
 */
mu_ent *new_ent(apr_pool_t *pool, mu_ent *head)
{
	mu_ent *p = (mu_ent *)ap_pcalloc(pool, sizeof(mu_ent));
	p->next = head;
	return p;
}

/**
 * Add a file (and the content of the directory if requested) to the chain.
 *
 * This function creates a new entry from a file.
 * If the file is a directory and the recursive option is set, all its content
 * is also added.
 * If the file is an ogg vorbis or an mp3, it is simply added.
 * 
 * @param pool Pool
 * @param r Apache request_rec struct to handle log writings (debugging)
 * @param head Head
 * @param conf MusicIndex configuration paramaters struct
 * @param names Names
 *
 * @return When possible, struct mu_ent correctly set up
 */
static mu_ent *make_music_entry(apr_pool_t *pool, request_rec *r,
	mu_ent *head, mu_config *conf, mu_ent_names *names)
{
	DIR 			*dir;
	struct dirent		*dstruct;
	mu_ent			*p = head;
	FILE			*in = NULL;
	unsigned short		i;
	char			*t, *fn, *uri;
	
	if (!names) {
		names = ap_palloc(r->pool, sizeof(mu_ent_names));
		strcpy(names->filename, r->filename);
		strcpy(names->uri, r->parsed_uri.path);
		names->create_cache_file = 0;
	}

	t = names->uri + strlen(names->uri);

	while (*(--t) != '/');
	if (*(++t) == '.')	/* we don't want "invisible" files to show */	
		return p;

	if ((in = fopen(names->filename, "r"))) {
		if ((ap_is_directory(pool, names->filename))) {
			fn = names->filename + strlen(names->filename);
			*fn++ = '/';
			*fn = '\0';
		
			uri = names->uri + strlen(names->uri) - 1;
			if (*uri++ != '/')
				*uri++ = '/';
			*uri = '\0';
				
			if (conf->options & MI_RECURSIVE) {
				conf->options &= (conf->play_recursive);

				if (!(dir = opendir(names->filename)))
					return head;

				if ((conf->cache_path) && (cache_check_dir(r, conf, names->filename)))
					ap_log_rerror(APLOG_MARK, APLOG_ERR, r, "[musicindex] DBG: cache_check_dir failed");	/* XXX debug */

				while ((dstruct = readdir(dir))) {
					strcpy(fn, dstruct->d_name);
					strcpy(uri, dstruct->d_name);
					p = make_music_entry(pool, r, p, conf, names);	
				}
				closedir(dir);
				fclose(in);
				return p;
			}
			else if (!(conf->options & MI_PLAYLIST)) {
				p = new_ent(pool, head);
				p->filetype = FT_DIR;
				fclose(in);
			}
			else {
				fclose(in);
				return p;
			}
		}

		/* The end of adventures for the array of function pointers.
		   We call them sequentially, until a function has recognised
		   it (p!=head), or until we reach the end of the array. */
		for (i=0; (make_entry[i] != NULL) && (p == head); i++)
			p = make_entry[i](pool, head, in, conf, names, r);
		
		/* File not recognised, so not closed. Do it now! */
		if (p == head)
			fclose(in);
	}

	if (p == head) {
		names->create_cache_file = 0;
		return p;
	}

	p->uri = ap_pstrdup(pool, names->uri);
	p->file = p->uri + strlen(r->parsed_uri.path);
	
	if (p->filetype == FT_DIR) {
		names->create_cache_file = 0;
		return p;
	}

	if (!(p->title)) {
#ifdef NO_TITLE_STRIP
		p->title = p->file;
#else
		/* Copy the name removing file extension and changing '_' to ' ' */
		p->title = ap_pstrndup(pool, p->file, strlen(p->file) - 4);
		for (i=0; p->title[i] != '\0'; i++)
			if (p->title[i] == '_')
				p->title[i] = ' ';
#endif	/* NO_TITLE_STRIP */
		/* We remove the trailing path if any. basename() ? */
		if (strrchr(p->title, '/')) {
			p->title = strrchr(p->title, '/');
			p->title++;
		}

	}
	
	/* We put that here so that we do not create cache files for unhandled file types,
	 * and p->title is formated as we desire */
	if (names->create_cache_file)
		cache_write_file(r, p, conf, names);
	
	if (!(conf->options & MI_SEARCH))
		return p;

	/* Here we go if we have an ongoing search request.
	 * We will return only the entries matching the search criteria */
	if (p->file && (ap_strcasestr(p->file, conf->search)))
		return p;
	if (p->artist && (ap_strcasestr(p->artist, conf->search)))
		return p;
	if (p->album && (ap_strcasestr(p->album, conf->search)))
		return p;
	if (p->title && (ap_strcasestr(p->title, conf->search)))
		return p;

	return head;
}

/**
 * Creates and sends a playlist or webpage with the content of a directory.
 *
 * This function generates a list containing all the files in the current
 * directory, either recursively or not. The list is then sent, either as a
 * m3u playlist or a web page.
 * 
 * @param r Apache request_rec struct
 * @param conf MusicIndex configuration paramaters struct
 *
 * @return 0 on success, error code otherwise.
 */
short musicindex_list(request_rec *r, mu_config *conf)
{
	apr_dir_t	*dir;
	mu_ent		*head = NULL;

	apr_dir_open(&dir, r->filename, r->pool);

	if (dir == NULL) {
		ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
			"Can't open directory for music index: %s",
			r->filename);
		return HTTP_FORBIDDEN;
	}

	ap_pclosedir(r->pool, dir);

	if (conf->options & MI_PLAYLIST) {
		r->content_type = "audio/mpegurl";
		ap_table_setn(r->headers_out, "Content-Disposition",
			"filename = \"playlist.m3u\"");
	}
	else
		r->content_type = "text/html; charset=UTF-8";
	
	ap_send_http_header(r);

	if (r->header_only)
		return 0;
	
	ap_hard_timeout("send music list", r);

	head = make_music_entry(r->pool, r, NULL, conf, NULL);
	head = quicksort(head, NULL, conf);

	if (conf->options & MI_PLAYLIST)
		send_playlist(r, head, conf);
	else {
		send_head(r, conf);
		if (!(conf->options & MI_SEARCH))
			send_directories(r, head, conf);
		send_tracks(r, head, conf);
		send_foot(r, conf);
	}

	ap_kill_timeout(r);
	return 0;
}

/**
 * Generates and sends a playlist with only selected items.
 *
 * This function parses the requests, which is the form :
 * http://...../?file=file1&file=file2&file=file3...
 *
 * Note that in FORM requests, arguments have ' ' replaced by '+', other
 * "special" characters are changed to their URI equivalent. Here is the trick :
 * ap_unescape_url doesn't change the '+' to its URI equivalent. So we first
 * change ' ' to '+' and then call the apache function. Easy, isn't it ?
 * 
 * @param r Apache request_rec struct
 * @param conf MusicIndex configuration paramaters struct
 *
 * @return 0
 */
short playlist_selected(request_rec *r, mu_config *conf)
{
	const char		*args = r->args;
	char			*p;

	char			*fn, *uri;
	mu_ent_names		*names;
	mu_ent			*head = NULL;

	r->content_type = "audio/mpegurl";
	ap_table_setn(r->headers_out, "Content-Disposition",
		"filename = \"playlist.m3u\"");

	ap_send_http_header(r);

	if (r->header_only)
		return 0;
	ap_hard_timeout("send playlist", r);

	if ((args == NULL) || (*args == '\0')) {
		ap_kill_timeout(r);
		return 0;
	}

	names = ap_palloc(r->pool, sizeof(mu_ent_names));
	strcpy(names->filename, r->filename);
	strcpy(names->uri, r->parsed_uri.path);
	names->create_cache_file = 0;

	fn = names->filename + strlen(names->filename);
	*fn++ = '/';
	*fn = '\0';

	uri = names->uri + strlen(names->uri) - 1;
	if (*uri++ != '/')
		*uri++ = '/';
	*uri = '\0';

	while (args[0]) {
		p = ap_getword(r->pool, &args, '&');
		if (!strncmp(p, "file=", 5))
		{
			unsigned short i;
			p += 5;
			strcpy(fn, p);
			for (i=0; fn[i]; i++) {
				if (fn[i] == '+')
					fn[i] = ' ';
			}
			ap_unescape_url(fn);
			strcpy(uri, fn);
			head = make_music_entry(r->pool, r, head, conf, names);
		}
	}

	head = quicksort(head, NULL, conf);
	send_playlist(r, head, conf);

	ap_kill_timeout(r);
	return 0;
}

/**
 * Generates a playlist for a single file
 *
 * This function takes the requested filename from the request and sends
 * the http link in a "audio/mpegurl" (aka playlist) file.
 * 
 * @param r Apache request_rec struct
 * @param conf MusicIndex configuration paramaters struct
 *
 * @return 0
 */
short playlist_single(request_rec *r, mu_config *conf)
{
	mu_ent			*head = NULL;

	r->content_type = "audio/mpegurl";
	ap_table_setn(r->headers_out, "Content-Disposition",
		"filename = \"playlist.m3u\"");

	ap_send_http_header(r);

	if (r->header_only)
		return 0;
	
	ap_hard_timeout("send playlist", r);

	head = make_music_entry(r->pool, r, head, conf, NULL);
	send_playlist(r, head, conf);
	
	ap_kill_timeout(r);
	return 0;
}
